package com.fr.plugin.db.redis.core.order.impl;

import com.fr.base.BaseFormula;
import com.fr.base.Parameter;
import com.fr.base.ParameterHelper;
import com.fr.base.ParameterMapNameSpace;
import com.fr.general.GeneralUtils;
import com.fr.log.FineLoggerFactory;
import com.fr.plugin.db.redis.core.order.OrderValue;
import com.fr.script.Calculator;
import com.fr.stable.ParameterProvider;
import com.fr.stable.StringUtils;
import com.fr.stable.UtilEvalError;
import com.fr.stable.script.NameSpace;
import com.fr.stable.xml.XMLableReader;

public class FormulaOrderValue extends OrderValue {

    private BaseFormula formula;

    public FormulaOrderValue() {

    }

    public FormulaOrderValue(BaseFormula formula) {
        this.formula = formula;
    }

    public void setValue(BaseFormula formula) {
        this.formula = formula;
    }

    @Override
    public BaseFormula getValue() {
        return formula;
    }

    @Override
    public int toIndex(Calculator cal, Parameter[] ps) {
        NameSpace nameSpace = ParameterMapNameSpace.create(ps);
        cal.pushNameSpace(nameSpace);
        try {
            return GeneralUtils.objectToNumber(formula.eval(cal)).intValue();
        } catch (UtilEvalError u) {
            FineLoggerFactory.getLogger().error(u.getMessage(), u);
        } finally {
            cal.removeNameSpace(nameSpace);
        }
        return 0;
    }

    @Override
    public ParameterProvider[] analyze4Parameters() {
        if (formula == null) {
            return new ParameterProvider[0];
        }
        return ParameterHelper.analyze4ParametersFromFormula(formula.getContent());
    }

    @Override
    public void readValue(XMLableReader reader) {
        this.formula = BaseFormula.createFormulaBuilder().build(reader.getAttrAsString("value", StringUtils.EMPTY));
    }

    @Override
    public String toString() {
        return formula == null ? StringUtils.EMPTY : formula.getPureContent();
    }
}
