package com.fr.plugin.db.redis.ui;

import com.fr.design.border.UITitledBorder;
import com.fr.design.data.datapane.connect.DatabaseConnectionPane;
import com.fr.design.dialog.BasicDialog;
import com.fr.design.dialog.DialogActionAdapter;
import com.fr.design.gui.ibutton.UIPasswordField;
import com.fr.design.gui.ilable.ActionLabel;
import com.fr.design.gui.ilable.UILabel;
import com.fr.design.gui.itextfield.UIIntNumberField;
import com.fr.design.gui.itextfield.UINumberField;
import com.fr.design.gui.itextfield.UITextField;
import com.fr.design.i18n.Toolkit;
import com.fr.design.layout.FRGUIPaneFactory;
import com.fr.design.layout.TableLayout;
import com.fr.design.layout.TableLayoutHelper;
import com.fr.design.utils.gui.GUICoreUtils;
import com.fr.plugin.db.redis.core.RedisDatabaseConnection;
import com.fr.plugin.db.redis.core.pool.RedisConnectionPoolConfig;
import com.fr.plugin.db.redis.core.pool.RedisConnectionProxyConfig;
import com.fr.plugin.db.redis.ui.pool.RedisConnectionPoolConfigPane;
import com.fr.plugin.db.redis.ui.proxy.RedisConnectionProxyConfigPane;
import com.fr.transaction.Configurations;
import com.fr.transaction.WorkerFacade;

import javax.swing.*;
import java.awt.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;


public class RedisConnectionPane extends DatabaseConnectionPane<com.fr.plugin.db.redis.core.RedisDatabaseConnection> {

    private UITextField hostTextField;
    private UIIntNumberField portNumberField;
    private UIPasswordField passwordTextField;



    @Override
    protected JPanel mainPanel() {
        JPanel pane = new JPanel();
        pane.setLayout(new BorderLayout());
        hostTextField = new UITextField();
        portNumberField = new UIIntNumberField();
        passwordTextField = new UIPasswordField();


        JPanel globalConfigPane = new JPanel();
        GridLayout gridLayout = new GridLayout(1, 2);
        globalConfigPane.setLayout(gridLayout);

        Component[][] components = new Component[][]{
                {new UILabel(Toolkit.i18nText("Plugin-Redis_Host") + ":"), hostTextField},
                {new UILabel(Toolkit.i18nText("Plugin-Redis_Port") + ":"), portNumberField},
                {new UILabel(Toolkit.i18nText("Plugin-Redis_Password") + ":"), passwordTextField},
                {null, globalConfigPane}
        };

        ActionLabel poolConfigLabel = new ActionLabel(Toolkit.i18nText("Plugin-Redis_Pool_Config"));
        ActionLabel proxyConfigLabel = new ActionLabel(Toolkit.i18nText("Plugin-Redis_Proxy_Config"));
        poolConfigLabel.addActionListener(new ActionListener() {
            @Override
            public void actionPerformed(ActionEvent e) {
                final RedisConnectionPoolConfig poolConfig = RedisConnectionPoolConfig.getInstance();
                final RedisConnectionPoolConfigPane poolConfigPane = new RedisConnectionPoolConfigPane();
                poolConfigPane.populate(poolConfig);
                BasicDialog dialog = poolConfigPane.showWindow(SwingUtilities.getWindowAncestor(RedisConnectionPane.this), new DialogActionAdapter() {
                    @Override
                    public void doOk() {
                        Configurations.modify(new WorkerFacade(RedisConnectionPoolConfig.class) {
                            @Override
                            public void run() {
                                poolConfigPane.update(poolConfig);
                            }
                        });
                    }
                });
                dialog.setVisible(true);
            }
        });
        proxyConfigLabel.addActionListener(new ActionListener() {
            @Override
            public void actionPerformed(ActionEvent e) {
                final RedisConnectionProxyConfig proxyConfig = RedisConnectionProxyConfig.getInstance();
                final RedisConnectionProxyConfigPane proxyConfigPane = new RedisConnectionProxyConfigPane();
                proxyConfigPane.populate(proxyConfig);
                BasicDialog dialog = proxyConfigPane.showWindow(SwingUtilities.getWindowAncestor(RedisConnectionPane.this), new DialogActionAdapter() {
                    @Override
                    public void doOk() {
                        Configurations.modify(new WorkerFacade(RedisConnectionProxyConfig.class) {
                            @Override
                            public void run() {
                                proxyConfigPane.update(proxyConfig);
                            }
                        });
                    }
                });
                dialog.setVisible(true);
            }
        });
        globalConfigPane.add(GUICoreUtils.createFlowPane(poolConfigLabel, FlowLayout.LEFT));
        globalConfigPane.add(GUICoreUtils.createFlowPane(proxyConfigLabel, FlowLayout.LEFT));
        double p = TableLayout.PREFERRED;

        double[] rowSize = new double[]{p, p, p, p, p};
        double[] columnSize = new double[]{p, 400};

        JPanel settingsUI = TableLayoutHelper.createTableLayoutPane(components, rowSize, columnSize);
        settingsUI.setBorder(UITitledBorder.createBorderWithTitle("Redis"));

        JPanel centerPane = FRGUIPaneFactory.createNormalFlowInnerContainer_M_Pane();

        centerPane.add(settingsUI);

        pane.add(centerPane, BorderLayout.CENTER);
        return pane;
    }

    @Override
    protected boolean isFineBI() {
        return false;
    }

    @Override
    protected void populateSubDatabaseConnectionBean(RedisDatabaseConnection ob) {
        hostTextField.setText(ob.getHost());
        portNumberField.setValue(ob.getPort());
        passwordTextField.setText(ob.getPassword());
    }

    @Override
    protected com.fr.plugin.db.redis.core.RedisDatabaseConnection updateSubDatabaseConnectionBean() {
        RedisDatabaseConnection connection = new RedisDatabaseConnection();
        connection.setHost(hostTextField.getText());
        connection.setPort((int) portNumberField.getValue());
        connection.setPassword(passwordTextField.getText());
        return connection;
    }

    @Override
    protected String title4PopupWindow() {
        return "Redis";
    }
}