package com.fr.plugin.db.redis.ui.pool;

import com.fr.design.dialog.BasicPane;
import com.fr.design.gui.icheckbox.UICheckBox;
import com.fr.design.gui.ilable.UILabel;
import com.fr.design.gui.itextfield.UIIntNumberField;
import com.fr.design.gui.itextfield.UINumberField;
import com.fr.design.i18n.Toolkit;
import com.fr.design.layout.TableLayout;
import com.fr.design.layout.TableLayoutHelper;
import com.fr.plugin.db.redis.core.pool.RedisConnectionPoolConfig;

import javax.swing.*;
import java.awt.*;

/**
 * @author richie
 * @version 10.0
 * Created by richie on 2019-03-21
 */
public class RedisConnectionPoolConfigPane extends BasicPane {

    private UIIntNumberField maxTotalNumberFiled;
    private UINumberField maxWaitNumberField;
    private UIIntNumberField maxIdleNumberField;
    private UICheckBox blockWhenExhaustedCheckBox;
    private UICheckBox lifoCheckBox;
    private UIIntNumberField timeoutNumberField;

    public RedisConnectionPoolConfigPane() {
        setLayout(new BorderLayout());
        maxTotalNumberFiled = new UIIntNumberField();
        maxWaitNumberField = new UINumberField();
        maxWaitNumberField.setMinValue(-1);
        maxWaitNumberField.setMaxDecimalLength(0);

        maxIdleNumberField = new UIIntNumberField();
        blockWhenExhaustedCheckBox = new UICheckBox();
        blockWhenExhaustedCheckBox.setSelected(false);

        lifoCheckBox = new UICheckBox();

        timeoutNumberField = new UIIntNumberField();

        JComponent[][] comps = new JComponent[][]{
                {new UILabel(Toolkit.i18nText("Plugin-Redis_Pool_Max_Total") + ":"), maxTotalNumberFiled},
                {new UILabel(Toolkit.i18nText("Plugin-Redis_Pool_Max_Wait") + ":"), maxWaitNumberField},
                {new UILabel(Toolkit.i18nText("Plugin-Redis_Pool_Max_Idle") + ":"), maxIdleNumberField},
                {new UILabel(Toolkit.i18nText("Plugin-Redis_Pool_Block_When_Exhausted") + ":"), blockWhenExhaustedCheckBox},
                {new UILabel(Toolkit.i18nText("Plugin-Redis_Pool_Lifo") + ":"), lifoCheckBox},
                {new UILabel(Toolkit.i18nText("Plugin-Redis_Pool_Timeout") + ":"), timeoutNumberField}
        };
        double p = TableLayout.PREFERRED;
        double f = TableLayout.FILL;

        double[] rowSize = new double[]{p, p, p, p, p, p};
        double[] columnSize = new double[]{p, f};
        add(TableLayoutHelper.createTableLayoutPane(comps, rowSize, columnSize), BorderLayout.CENTER);
    }

    @Override
    protected String title4PopupWindow() {
        return Toolkit.i18nText("Plugin-Redis_Pool_Config");
    }

    public void populate(RedisConnectionPoolConfig config) {
        maxTotalNumberFiled.setValue(config.getMaxTotal());
        maxWaitNumberField.setValue(config.getMaxWait());
        maxIdleNumberField.setValue(config.getMaxIdle());
        blockWhenExhaustedCheckBox.setSelected(config.getBlockWhenExhausted());
        lifoCheckBox.setSelected(config.getLifo());
        timeoutNumberField.setValue(config.getTimeout());
    }

    public void update(RedisConnectionPoolConfig config) {
        config.setMaxTotal((int)maxTotalNumberFiled.getValue());
        config.setMaxWait((int) maxWaitNumberField.getValue());
        config.setMaxIdle((int) maxIdleNumberField.getValue());
        config.setBlockWhenExhausted(blockWhenExhaustedCheckBox.isSelected());
        config.setLifo(lifoCheckBox.isSelected());
        config.setTimeout((int) timeoutNumberField.getValue());
    }
}
