package com.fr.plugin.db.redis.ui.proxy;

import com.fr.design.dialog.BasicPane;
import com.fr.design.gui.ibutton.UIButton;
import com.fr.design.gui.icheckbox.UICheckBox;
import com.fr.design.gui.ilable.UILabel;
import com.fr.design.gui.ipasswordfield.UIPassWordField;
import com.fr.design.gui.itextfield.UIIntNumberField;
import com.fr.design.gui.itextfield.UITextField;
import com.fr.design.i18n.Toolkit;
import com.fr.design.layout.TableLayout;
import com.fr.design.layout.TableLayoutHelper;
import com.fr.design.utils.gui.GUICoreUtils;
import com.fr.plugin.db.redis.core.pool.RedisConnectionProxyConfig;

import javax.swing.*;
import java.awt.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.File;


/**
 * @author richie
 * @version 10.0
 * Created by richie on 2019-03-21
 */
public class RedisConnectionProxyConfigPane extends BasicPane {

    private UICheckBox openCheckBox;
    private UITextField hostTextField;
    private UIIntNumberField portNumberField;
    private UITextField usernameTextField;
    private UIPassWordField passwordTextField;
    private UITextField privateKeyPathTextField;

    public RedisConnectionProxyConfigPane() {
        setLayout(new BorderLayout());

        openCheckBox = new UICheckBox();
        openCheckBox.setSelected(false);

        hostTextField = new UITextField();

        portNumberField = new UIIntNumberField();
        usernameTextField = new UITextField();
        passwordTextField = new UIPassWordField();

        privateKeyPathTextField = new UITextField();
        privateKeyPathTextField.setPlaceholder(Toolkit.i18nText("Plugin-Redis_Proxy_Private_Key_Tip"));

        UIButton fileSelectButton = new UIButton("...");

        UILabel description = new UILabel(Toolkit.i18nText("Plugin-Redis_Proxy_Description"), SwingConstants.CENTER);

        JComponent[][] comps = new JComponent[][]{
                {new UILabel(Toolkit.i18nText("Plugin-Redis_Proxy_Open") + ":"), GUICoreUtils.createBorderLayoutPane(
                        openCheckBox, BorderLayout.WEST,
                        description, BorderLayout.CENTER
                )},
                {new UILabel(Toolkit.i18nText("Plugin-Redis_Proxy_Host") + ":"), hostTextField},
                {new UILabel(Toolkit.i18nText("Plugin-Redis_Proxy_Port") + ":"), portNumberField},
                {new UILabel(Toolkit.i18nText("Plugin-Redis_Proxy_Username") + ":"), usernameTextField},
                {new UILabel(Toolkit.i18nText("Plugin-Redis_Proxy_Password") + ":"), passwordTextField},
                {new UILabel(Toolkit.i18nText("Plugin-Redis_Proxy_Private_Key_Path") + ":"), GUICoreUtils.createBorderLayoutPane(
                        privateKeyPathTextField, BorderLayout.CENTER,
                        fileSelectButton, BorderLayout.EAST
                )},
        };
        fileSelectButton.addActionListener(new ActionListener() {
            @Override
            public void actionPerformed(ActionEvent e) {
                JFileChooser chooser = new JFileChooser();
                chooser.showOpenDialog(SwingUtilities.getWindowAncestor(RedisConnectionProxyConfigPane.this));
                File file = chooser.getSelectedFile();
                if (file != null) {
                    privateKeyPathTextField.setText(file.getAbsolutePath());
                }
            }
        });

        double p = TableLayout.PREFERRED;
        double f = TableLayout.FILL;

        double[] rowSize = new double[]{p, p, p, p, p, p};
        double[] columnSize = new double[]{p, f};
        add(TableLayoutHelper.createTableLayoutPane(comps, rowSize, columnSize), BorderLayout.CENTER);

    }

    @Override
    protected String title4PopupWindow() {
        return Toolkit.i18nText("Plugin-Redis_Proxy_Config");
    }

    public void populate(RedisConnectionProxyConfig config) {
        openCheckBox.setSelected(config.isOpen());
        hostTextField.setText(config.getHost());
        portNumberField.setValue(config.getPort());
        usernameTextField.setText(config.getUsername());
        passwordTextField.setText(config.getPassword());
        privateKeyPathTextField.setText(config.getPrivateKeyPath());
    }

    public void update(RedisConnectionProxyConfig config) {
        config.setOpen(openCheckBox.isSelected());
        config.setHost(hostTextField.getText());
        config.setPort((int) portNumberField.getValue());
        config.setUsername(usernameTextField.getText());
        config.setPassword(passwordTextField.getText());
        config.setPrivateKeyPath(privateKeyPathTextField.getText());
    }
}
